@@ -56,9 +56,9 @@ void on_cell_blocklist_edited (GtkCellRendererText *, gchar *, gchar *, gpointer
 void on_cell_messagelist_edited (GtkCellRendererText *, gchar *, gchar *, gpointer);
 void on_cell_messagelist_toggled (GtkCellRendererToggle *, gchar *, gpointer);
 void on_cell_preferences_midi_interfaces_toggled (GtkCellRendererToggle *, gchar *, gpointer);
-gboolean on_entry_instrumentproperties_name_changed(GtkEditable *, gpointer);
+void on_entry_instrumentproperties_name_changed(GtkEditable *, gpointer);
 void on_spinbutton_instrument_changed(GtkEditable *, gpointer);
-gboolean on_entry_songname_changed(GtkEditable *, gpointer);
+void on_entry_songname_changed(GtkEditable *, gpointer);
 void on_checkbutton_sendsync_toggled(GtkToggleButton *, gpointer);
 void on_scale_tempo_value_changed(GtkRange *, gpointer);
 void on_scale_tpl_value_changed(GtkRange *, gpointer);
