@@ -59,10 +59,8 @@ struct editor {
   GThread *thread;
   /* Mutex for the song */
   GMutex *songmutex;
-  /* Data structures of available MIDI interfaces */
-  struct midi **midiinterfaces;
-  /* Number of available MIDI interfaces */
-  unsigned int nummidiinterfaces;
+  /* MIDI subsystem */
+  struct midi *midi;
   /* Player for this editor */
   struct player *player;
   /* GConf client for preferences */
@@ -71,7 +69,6 @@ struct editor {
   /* Receive MIDI message status */
   unsigned char receiveactive;
   unsigned int receivemessage;
-  unsigned int receivemidiinterface;
   unsigned int receivedbytes;
   unsigned char receiveautostop;
 };
@@ -86,10 +83,8 @@ void editor_close(struct editor *);
 void editor_song_midi_interface_numbers_set(struct editor *);
 /* Remaps the instruments to current set of MIDI interfaces */
 void editor_song_midi_interface_names_set(struct editor *);
-/* Gets an MIDI interface number by name from the available interfaces */
-int editor_midi_interface_get_by_name(struct editor *, char *);
-/* Gets an MIDI interface name by number from the available interfaces */
-char *editor_midi_interface_name_get_by_number(struct editor *, unsigned int);
+/* Checks whether there have been changes in MIDI interfaces */
+void editor_midi_status_check(struct editor *);
 
 /* Refreshes playseq from section and block from position */
 void editor_refresh_playseq_and_block(struct editor *);
@@ -223,14 +218,12 @@ unsigned int editor_song_sendsync_get(struct editor *);
 void editor_scheduler_set(struct editor *, unsigned int);
 /* Returns the scheduling mode */
 unsigned int editor_scheduler_get(struct editor *);
-/* Sets the name of the MIDI interface to use */
-void editor_midi_interface_set(struct editor *, char *, unsigned int, unsigned int);
+/* Enables or disables a MIDI interface by name */
+unsigned int editor_midi_interface_status_set(struct editor *, unsigned int, unsigned int, unsigned int, unsigned int);
 /* Returns the name of the MIDI interface to use */
 char *editor_midi_interface_get(struct editor *, unsigned int, unsigned int);
 /* Marks the enabled interfaces as enabled in the preferences */
 void editor_midi_interface_set_from_current(struct editor *);
-/* Enables or disables a MIDI interface by name */
-unsigned int editor_midi_interface_status_set(struct editor *, char *, unsigned int, unsigned int);
 /* Returns the song path from the preferences */
 char *editor_song_path_get(struct editor *);
 /* Returns the message path from the preferences */
