@@ -1,6 +1,6 @@
 /* instrument.h
  *
- * Copyright 2002-2004 Vesa Halttunen
+ * Copyright 2002-2005 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -24,9 +24,11 @@
 
 #include <libxml/tree.h>
 
-struct instrument {
-  /* Name */
-  char *name;
+struct instrument_output {
+  /* MIDI interface */
+  unsigned short midiinterface;
+  /* MIDI interface name (for remapping) */
+  char *midiinterfacename;
   /* MIDI preset */
   unsigned short midipreset;
   /* MIDI channel */
@@ -39,6 +41,15 @@ struct instrument {
   unsigned char hold;
 };
 
+struct instrument {
+  /* Name */
+  char *name;
+  /* Outputs of the instrument */
+  struct instrument_output **outputs;
+  /* Number of outputs */
+  unsigned int numoutputs;
+};
+
 struct instrument *instrument_alloc();
 void instrument_free(struct instrument *);
 /* Parses an instrument element in an XML file */
