@@ -46,6 +46,7 @@ struct player_track_status {
   char instrument;
   char previouscommand;
   struct player_interface_status **interfaces;
+  int numinterfaces;
 };
 
 /* 128 MIDI controllers plus aftertouch, channel pressure and pitch wheel */
@@ -75,6 +76,8 @@ struct player {
   struct player_track_status **trackstatus;
   /* MIDI controller values; one for each controller on each channel */
   unsigned char **midicontrollervalues;
+  /* Number of entries in midicontrollervalues */
+  int nummidicontrollervalues;
   /* For measuring how long the song has been playing */
   struct timeval playingstarted, playedsofar;
   /* Ticks passed after playing started */
@@ -85,10 +88,8 @@ struct player {
   GMutex *mutex;
   /* Kill player flag (the mutex must be used when accessing) */
   unsigned int killthread;
-  /* MIDI data structures */
-  struct midi **midiinterfaces;
-  /* Number of MIDI data structures */
-  unsigned int nummidiinterfaces;
+  /* MIDI subsystem */
+  struct midi *midi;
   /* RTC device file descriptor */
   int rtc;
   /* Editor for this player */
@@ -98,9 +99,9 @@ struct player {
 };
 
 /* Plays a song to a MIDI interface witout any scheduling (for export) */
-void player_midi_export(struct song *, struct midi *);
+void player_midi_export(struct song *, struct midi_interface *);
 /* Creates a player for a song */
-struct player *player_open(struct song *, struct editor *, struct midi **, unsigned int, unsigned int);
+struct player *player_open(struct song *, struct editor *, struct midi *, unsigned int);
 /* Closes a player for a song */
 void player_close(struct player *);
 
@@ -133,5 +134,12 @@ void player_set_line(struct player *, int);
 void player_set_solo(struct player *, unsigned int);
 /* Checks whether some tracks are soloed or not */
 void player_check_solo(struct player *);
+/* Notifies the player that MIDI interfaces have changed */
+void player_midi_changed(struct player *);
+
+/* Lock the player */
+void player_lock(struct player *);
+/* Unlock the player */
+void player_unlock(struct player *);
 
 #endif
