@@ -783,7 +783,7 @@ void on_button_message_receive_clicked(GtkButton * button, gpointer user_data)
   gui->editor->receivemessage = gui->messagelist_message;
   
   gtk_window_present(GTK_WINDOW(window));
-  gui_receive_refresh(gui, 0, 0, TRUE);
+  gui_receive_refresh(gui, -1, 0);
 }
 
 void on_button_receive_clicked(GtkButton * button, gpointer user_data)
@@ -791,10 +791,9 @@ void on_button_receive_clicked(GtkButton * button, gpointer user_data)
   struct gui *gui = (struct gui *)user_data;
   GtkWindow *window = GTK_WINDOW(glade_xml_get_widget(gui->xml, "dialog_receive"));
 
-  if (!gui->editor->receiveactive && gui->editor->receivemidiinterface != -1) {
+  if (!gui->editor->receiveactive) {
     /* Start receiving */
     gtk_window_set_modal(window, TRUE);
-    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "combobox_receive"), FALSE);
     gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "checkbutton_receive_autostop"), FALSE);
     gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_receive"), FALSE);
     gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_receive_close"), FALSE);
@@ -816,7 +815,6 @@ void on_button_receive_stop_clicked(GtkButton * button, gpointer user_data)
   if (gui->editor->receiveactive) {
     /* Stop receiving */
     gtk_window_set_modal(window, FALSE);
-    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "combobox_receive"), TRUE);
     gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "checkbutton_receive_autostop"), TRUE);
     gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_receive"), TRUE);
     gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_receive_close"), TRUE);
@@ -1043,14 +1041,12 @@ void on_button_blocklist_delete_clicked(GtkButton * button, gpointer user_data)
   gui_tracker_refresh(gui);
 }
 
-gboolean on_entry_instrumentproperties_name_changed(GtkEditable * widget, gpointer user_data)
+void on_entry_instrumentproperties_name_changed(GtkEditable * widget, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
 
   editor_song_instrument_current_name_set(gui->editor, (char *)gtk_entry_get_text(GTK_ENTRY(widget)));
   gui_instrument_refresh(gui);
-
-  return FALSE;
 }
 
 void on_button_messagelist_close_clicked(GtkButton * button, gpointer user_data)
@@ -1098,14 +1094,12 @@ void on_button_file_saveas_cancel_clicked(GtkButton * button, gpointer user_data
     gtk_widget_hide(fs);
 }
 
-gboolean on_entry_songname_changed(GtkEditable * widget, gpointer user_data)
+void on_entry_songname_changed(GtkEditable * widget, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
 
   editor_song_name_set(gui->editor, (char *)gtk_entry_get_text(GTK_ENTRY(widget)));
   gui_window_title_refresh(gui);
-
-  return FALSE;
 }
 
 void on_button_songproperties_close_clicked(GtkButton * button, gpointer user_data)
@@ -1889,15 +1883,13 @@ void on_cell_blocklist_edited(GtkCellRendererText * cell, gchar * path_string, g
     }
   case GUI_BLOCKLIST_COLUMN_TRACKS:
     {
-      int oldmax = editor_song_maxtracks_get(gui->editor);
-
       editor_song_block_current_tracks_set(gui->editor, value);
       gui_tracker_refresh(gui);
 
-      if (editor_song_maxtracks_check(gui->editor)) {
-	editor_player_trackstatus_create(gui->editor, oldmax);
+      /* No need to call editor_player_trackstatus_create since
+	 editor_song_maxtracks_check does the necessary thing automatically */
+      if (editor_song_maxtracks_check(gui->editor))
 	gui_trackvolumes_refresh(gui);
-      }
       break;
     }
   case GUI_BLOCKLIST_COLUMN_LENGTH:
@@ -1996,27 +1988,22 @@ void on_cell_preferences_midi_interfaces_toggled(GtkCellRendererToggle * cell, g
   GtkTreeView *treeview;
   GtkTreeModel *model;
   GtkTreeIter iter;
-  guint column;
+  guint column, direction;
   gboolean state;
-  GValue device = { 0 };
-  unsigned int direction;
-
-  /* Column number is passed as renderer object data */
+  GValue client = { 0 }, port = { 0 };
+  
+  /* Column number and direction are passed as renderer object data */
   gpointer columnptr = g_object_get_data(G_OBJECT(cell), "column");
-
-  if (columnptr == NULL)
-    return;
+  gpointer directionptr = g_object_get_data(G_OBJECT(cell), "direction");
 
   column = GPOINTER_TO_UINT(columnptr);
-
-  /* Get direction */
-  if (column == GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_INPUT_ENABLED)
-    direction = DIRECTION_IN;
-  else
-    direction = DIRECTION_OUT;
+  direction = GPOINTER_TO_UINT(directionptr);
 
   /* Get the iterator */
-  treeview = GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "treeview_preferences_midi_interfaces"));
+  if (direction == DIRECTION_OUT)
+    treeview = GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "treeview_preferences_midi_interfaces_output"));
+  else
+    treeview = GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "treeview_preferences_midi_interfaces_input"));    
   model = gtk_tree_view_get_model(treeview);
   gtk_tree_model_get_iter_from_string(model, &iter, path_string);
 
@@ -2027,8 +2014,9 @@ void on_cell_preferences_midi_interfaces_toggled(GtkCellRendererToggle * cell, g
   else
     state = TRUE;
 
-  gtk_tree_model_get_value(model, &iter, GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_DEVICENAME, &device);
-  if (editor_midi_interface_status_set(gui->editor, (char *)g_value_get_string(&device), direction, state))
+  gtk_tree_model_get_value(model, &iter, GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_CLIENT, &client);
+  gtk_tree_model_get_value(model, &iter, GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_PORT, &port);
+  if (editor_midi_interface_status_set(gui->editor, g_value_get_int(&client), g_value_get_int(&port), direction, state))
     state = TRUE;
   else
     state = FALSE;
@@ -2036,32 +2024,29 @@ void on_cell_preferences_midi_interfaces_toggled(GtkCellRendererToggle * cell, g
   /* Update the model according to the true status of the device */
   gtk_list_store_set(GTK_LIST_STORE(model), &iter, column, state, -1);
 
-  g_value_unset(&device);
+  g_value_unset(&client);
+  g_value_unset(&port);
 }
 
 void on_combobox_instrumentproperties_midiinterface_changed(GtkComboBox * combobox, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
+  GtkTreeModel *model;
+  GtkTreeIter iter;
+  GValue interface = { 0 };
+  int i = 0;
 
-  editor_song_instrument_current_midiinterface_set(gui->editor, (unsigned short)gtk_combo_box_get_active(combobox));
+  if (gtk_combo_box_get_active_iter(combobox, &iter)) {
+    model = gtk_combo_box_get_model(combobox);
+    gtk_tree_model_get_value(model, &iter, 1, &interface);
+    i = g_value_get_int(&interface);
+    g_value_unset(&interface);
+  }
+
+  editor_song_instrument_current_midiinterface_set(gui->editor, i);
 }
 
 void on_dialog_receive_delete_event(GtkWidget * widget, GdkEvent * event, gpointer user_data)
 {
   on_button_receive_close_clicked(NULL, user_data);
 }
-
-void on_combobox_receive_changed(GtkComboBox * combobox, gpointer user_data)
-{
-  struct gui *gui = (struct gui *)user_data;
-  GtkTreeModel *model;
-  GtkTreeIter iter;
-  GValue interface = { 0 };
-
-  if (gtk_combo_box_get_active_iter(combobox, &iter)) {
-    model = gtk_combo_box_get_model(combobox);
-    gtk_tree_model_get_value(model, &iter, 1, &interface);
-    gui->editor->receivemidiinterface = g_value_get_int(&interface);
-    g_value_unset(&interface);
-  }
-}
