@@ -553,7 +553,7 @@ struct smf *song_convert_smf(struct song *song)
   nullcheck_pointer(song, song_convert_smf);
 
   struct smf *smf;
-  struct midi *midi;
+  struct midi_interface *midi;
   int i;
   unsigned char endMTrk[3];
   
@@ -580,8 +580,7 @@ struct smf *song_convert_smf(struct song *song)
     smf->MTrks[i]->ID[3] = 'k';
   }
   
-  midi = midi_alloc(MIDI_BUFFER, NULL, NULL);
-  midi_open(midi, DIRECTION_BOTH);
+  midi = midi_interface_alloc(MIDI_BUFFER, NULL);
   player_midi_export(song, midi);
   endMTrk[0] = 0xff;
   endMTrk[1] = 0x2f;
@@ -589,7 +588,7 @@ struct smf *song_convert_smf(struct song *song)
   midi_write_raw(midi, endMTrk, 3);
   smf->MTrks[0]->Length = midi_get_buffer_length(midi);
   smf->MTrks[0]->Data = midi_get_buffer(midi);
-  midi_free(midi);
+  midi_interface_free(midi);
   
   return smf;
 }
