@@ -1,9 +1,3 @@
-/*
- * - initissä yritetään avata sekä in/out että unavailablein/out
- * - onnistuneet -> in/out
- * - haluttujen ja onnistuneiden erotus -> unavailablein/out
- * - haluttaessa avata jotain jos onnistuu -> in/out, muutoin unavailablein/out
- */
 /* editor.c
  *
  * Copyright 2002-2005 Vesa Halttunen
@@ -34,24 +28,17 @@
 #include "player.h"
 #include "callbacks.h"
 
-static char *string_remove_string(char *from, char *toremove);
-static char *string_append_char(char *to, char append);
-static char *string_append_string(char *to, char *append);
-
 /* Initializes the editor */
 struct editor *editor_open()
 {
   struct editor *editor = (struct editor *)calloc(1, sizeof(struct editor));
 
-  int i;
-
   /* Use GConf */
   editor->gconfclient = gconf_client_get_default();
   gconf_client_add_dir(editor->gconfclient, "/apps/tutka", GCONF_CLIENT_PRELOAD_NONE, NULL);
 
   /* Set defaults */
   editor->space = 1;
-  editor->receivemidiinterface = -1;
 
   /* Get a pointer to the main thread and lower priority to give time to the player */
   editor->thread = g_thread_self();
@@ -67,19 +54,8 @@ struct editor *editor_open()
     return NULL;
   }
 
-  /* Get MIDI interfaces */
-  editor->nummidiinterfaces = midi_get_num_alsa_devices();
-  editor->midiinterfaces = midi_get_alsa_devices();
-
-  /* Add a null MIDI interface */
-  editor->nummidiinterfaces++;
-  editor->midiinterfaces = (struct midi **)realloc(editor->midiinterfaces, editor->nummidiinterfaces * sizeof(struct midi *));
-  for (i = editor->nummidiinterfaces - 1; i > 0; i--)
-    editor->midiinterfaces[i] = editor->midiinterfaces[i - 1];
-  editor->midiinterfaces[0] = midi_alloc(MIDI_NULL, NULL, "No output");
-
-  /* Open the MIDI interfaces defined in preferences */
-  midi_open_defined(editor->midiinterfaces, editor->nummidiinterfaces, editor_midi_interface_get(editor, DIRECTION_IN, MIDIINTERFACES_AVAILABLE), editor_midi_interface_get(editor, DIRECTION_OUT, MIDIINTERFACES_AVAILABLE), editor_midi_interface_get(editor, DIRECTION_IN, MIDIINTERFACES_UNAVAILABLE), editor_midi_interface_get(editor, DIRECTION_OUT, MIDIINTERFACES_UNAVAILABLE));
+  /* Open the MIDI subsystem */
+  editor->midi = midi_open(editor_midi_interface_get(editor, DIRECTION_IN, MIDIINTERFACES_AVAILABLE), editor_midi_interface_get(editor, DIRECTION_OUT, MIDIINTERFACES_AVAILABLE), editor_midi_interface_get(editor, DIRECTION_IN, MIDIINTERFACES_UNAVAILABLE), editor_midi_interface_get(editor, DIRECTION_OUT, MIDIINTERFACES_UNAVAILABLE));
 
   /* Some interfaces may have been removed so refresh preferences */
   editor_midi_interface_set_from_current(editor);
@@ -134,7 +110,7 @@ void editor_song_open(struct editor *editor, char *filename)
     if (editor->song == NULL) {
       /* Figure out default MIDI interface number for new instruments */
       int i = 0;
-      if (editor->nummidiinterfaces > 1)
+      if (editor->midi->numoutputs > 1)
 	i = 1;
 
       editor->song = song_alloc();
@@ -152,7 +128,7 @@ void editor_song_open(struct editor *editor, char *filename)
   } else {
     /* Figure out default MIDI interface number for new instruments */
     int i = 0;
-    if (editor->nummidiinterfaces > 1)
+    if (editor->midi->numoutputs > 1)
       i = 1;
 
     editor->filename = NULL;
@@ -170,7 +146,7 @@ void editor_song_open(struct editor *editor, char *filename)
   editor->cmdpage = 0;
 
   /* Open the player */
-  editor->player = player_open(editor->song, editor, editor->midiinterfaces, editor->nummidiinterfaces, editor_scheduler_get(editor));
+  editor->player = player_open(editor->song, editor, editor->midi, editor_scheduler_get(editor));
 
   /* Refresh the GUI */
   gui_refresh_all(editor->gui);
@@ -182,18 +158,13 @@ void editor_close(struct editor *editor)
 {
   nullcheck_void(editor, editor_close);
 
-  int i;
-
   /* Close the player */
   if (editor->player != NULL)
     player_close(editor->player);
-
+  
   /* Close MIDI */
-  if (editor->midiinterfaces != NULL) {
-    for (i = 0; i < editor->nummidiinterfaces; i++)
-      midi_free(editor->midiinterfaces[i]);
-    free(editor->midiinterfaces);
-  }
+  if (editor->midi != NULL)
+    midi_close(editor->midi);
 
   /* Remove the GConf notification */
   if (editor->gconfclient != NULL)
@@ -244,7 +215,7 @@ void editor_song_midi_interface_numbers_set(struct editor *editor)
     struct instrument *instrument = editor->song->instruments[i];
     for (j = 0; j < instrument->numoutputs; j++) {
       struct instrument_output *output = instrument->outputs[j];
-      output->midiinterface = editor_midi_interface_get_by_name(editor, output->midiinterfacename);
+      output->midiinterface = midi_interface_number_get_by_name(editor->midi, output->midiinterfacename, DIRECTION_OUT);
     }
   }
 }
@@ -265,41 +236,44 @@ void editor_song_midi_interface_names_set(struct editor *editor)
       struct instrument_output *output = instrument->outputs[j];
       if (output->midiinterfacename != NULL)
 	free(output->midiinterfacename);
-      output->midiinterfacename = editor_midi_interface_name_get_by_number(editor, output->midiinterface);
+      output->midiinterfacename = midi_interface_name_get_by_number(editor->midi, output->midiinterface, DIRECTION_OUT);
     }
   }
 }
 
-/* Gets an MIDI interface number by name from the available interfaces */
-int editor_midi_interface_get_by_name(struct editor *editor, char *name)
-{
-  nullcheck_int(editor, editor_midi_interface_get_by_name);
+/* Checks whether there have been changes in MIDI interfaces */
+void editor_midi_status_check(struct editor *editor) {
+  nullcheck_void(editor, editor_midi_status_check);
+  
+  if(!editor->midi->changed)
+    return;
 
-  int i;
+  /* Store the old MIDI interface names */
+  editor_song_midi_interface_names_set(editor);
 
-  /* The first interface is always the null interface */
-  if (name == NULL)
-    return 0;
+  /* Make sure the player doesn't try to access the MIDI interfaces until they're refreshed */
+  player_lock(editor->player);
 
-  for (i = 1; i < editor->nummidiinterfaces; i++) {
-    struct midi *midi = editor->midiinterfaces[i];
-    if (strcmp(midi->devicename, name) == 0)
-      return i;
-  }
+  /* Refresh interfaces list */
+  midi_interfaces_refresh(editor->midi);
 
-  /* If no interface is found use the null interface */
-  return 0;
-}
+  /* Make sure the MIDI interface numbers are still valid */
+  editor_song_midi_interface_numbers_set(editor);
 
-/* Gets an MIDI interface name by number from the available interfaces */
-char *editor_midi_interface_name_get_by_number(struct editor *editor, unsigned int number)
-{
-  nullcheck_pointer(editor, editor_midi_interface_name_get_by_number);
+  /* Let the player know about the changes */
+  player_midi_changed(editor->player);
 
-  if (editor->midiinterfaces[number]->devicename != NULL)
-    return strdup(editor->midiinterfaces[number]->devicename);
-  else
-    return NULL;
+  /* The player is free to go */
+  player_unlock(editor->player);
+  
+  /* Refresh preferences and instrument windows */
+  gui_instrument_refresh(editor->gui);
+  gui_preferences_refresh(editor->gui);
+  
+  /* Update the preferences */
+  editor_midi_interface_set_from_current(editor);
+  
+  editor->midi->changed = 0;
 }
 
 /* Changes the current section and refreshes the GUI accordingly */
@@ -597,12 +571,9 @@ void editor_song_instrument_current_check(struct editor *editor)
 {
   nullcheck_void(editor, editor_song_instrument_current_check);
 
-  int i = 0;
-  if (editor->nummidiinterfaces > 1)
-    i = 1;
-
+  /* This routine COULD set the interface to the first subscribed interface... */
   g_mutex_lock(editor->songmutex);
-  song_check_instrument(editor->song, editor->instrument, i);
+  song_check_instrument(editor->song, editor->instrument, 0);
   g_mutex_unlock(editor->songmutex);
 }
 
@@ -1077,8 +1048,8 @@ void editor_song_message_send(struct editor *editor, unsigned int position)
 
   /* Should midi_write_raw be used here directly? */
   g_mutex_lock(editor->songmutex);
-  for (i = 1; i < editor->nummidiinterfaces; i++)
-    midi_write_raw(editor->midiinterfaces[i], editor->song->messages[position]->data, editor->song->messages[position]->length);
+  for (i = 1; i < editor->midi->numoutputs; i++)
+    midi_write_raw(editor->midi->outputs[i], editor->song->messages[position]->data, editor->song->messages[position]->length);
   g_mutex_unlock(editor->songmutex);
 }
 
@@ -1091,7 +1062,7 @@ unsigned int editor_song_message_receive(struct editor *editor)
     struct message *message = editor->song->messages[editor->receivemessage];
 
     /* Read as many SysEx bytes as available */
-    int i = midi_read_system_exclusive(editor->midiinterfaces[editor->receivemidiinterface], message, editor->receivedbytes, editor->receiveautostop);
+    int i = midi_read_system_exclusive(editor->midi, message, editor->receivedbytes, editor->receiveautostop);
 
     /* If the amount of bytes is negative last bytes have been received */
     if (i > 0)
@@ -1393,12 +1364,9 @@ void editor_song_instrument_check(struct editor *editor, unsigned int instrument
 {
   nullcheck_void(editor, editor_song_instrument_check);
 
-  int i = 0;
-  if (editor->nummidiinterfaces > 1)
-    i = 1;
-
+  /* This routine COULD check which one is the first subscribed MIDI interface... */
   g_mutex_lock(editor->songmutex);
-  song_check_instrument(editor->song, instrument, i);
+  song_check_instrument(editor->song, instrument, 0);
   g_mutex_unlock(editor->songmutex);
 }
 
@@ -1764,7 +1732,7 @@ unsigned int editor_scheduler_get(struct editor *editor)
   return scheduler;
 }
 
-/* Sets the names of the MIDI interfaces to enable separated by ; */
+/* Sets the names of the MIDI interfaces to enable separated by , */
 void editor_midi_interface_set(struct editor *editor, char *midi_interface, unsigned int direction, unsigned int mode)
 {
   nullcheck_void(editor, editor_midi_interface_set);
@@ -1782,7 +1750,7 @@ void editor_midi_interface_set(struct editor *editor, char *midi_interface, unsi
   }
 }
 
-/* Returns the names of the MIDI interfaces to enable separated by ; */
+/* Returns the names of the MIDI interfaces to enable separated by , */
 char *editor_midi_interface_get(struct editor *editor, unsigned int direction, unsigned int mode)
 {
   nullcheck_pointer(editor, editor_midi_interface_get);
@@ -1800,203 +1768,64 @@ char *editor_midi_interface_get(struct editor *editor, unsigned int direction, u
   }
 }
 
+/* Removes the last character from a string if the string is not null */
+static char *string_shorten(char *in) {
+  if (in == NULL)
+    return (char *)calloc(1, sizeof(char));
+  
+  int l = strlen(in);
+  if (l == 0)
+    return (char *)calloc(1, sizeof(char));
+    
+  char *out = (char *)calloc(l, sizeof(char));
+  memcpy(out, in, l - 1);
+  return out;
+}
+
 /* Marks the enabled interfaces as enabled in the preferences */
 void editor_midi_interface_set_from_current(struct editor *editor)
 {
   nullcheck_void(editor, editor_midi_interface_preferences_refresh);
 
-  int i, j, l, last;
-  char *instring = editor_midi_interface_get(editor, DIRECTION_IN, MIDIINTERFACES_AVAILABLE);
-  char *outstring = editor_midi_interface_get(editor, DIRECTION_OUT, MIDIINTERFACES_AVAILABLE);
-  char *unavailableinstring = editor_midi_interface_get(editor, DIRECTION_IN, MIDIINTERFACES_UNAVAILABLE);
-  char *unavailableoutstring = editor_midi_interface_get(editor, DIRECTION_OUT, MIDIINTERFACES_UNAVAILABLE);
-  char *current = NULL, *next = NULL;
-  unsigned int direction, unavailabledirection;
-
-  /* Add a ; after unavailable strings if they exist */
-  if (unavailableinstring != NULL && strlen(unavailableinstring) > 0)
-    unavailableinstring = string_append_char(unavailableinstring, ';');
-
-  if (unavailableoutstring != NULL && strlen(unavailableoutstring) > 0)
-    unavailableoutstring = string_append_char(unavailableoutstring, ';');
-
-  /* Move all the interfaces that do not exist to unavailable */
-  for (j = 0; j < 2; j++) {
-    last = 0;
-
-    switch (j) {
-    case 0:
-      current = instring;
-      next = instring;
-      direction = DIRECTION_IN;
-      break;
-    case 1:
-      current = outstring;
-      next = outstring;
-      direction = DIRECTION_OUT;
-      break;
-    }
-
-    /* If there are no devices to go through do nothing */
-    if (strlen(current) == 0)
-      last = 1;
-    
-    while (!last) {
-      /* Find the end of the device name */
-      while (*next != ';' && *next != 0)
-	next++;
-      if (*next == 0)
-        last = 1;
-      else
-	*next = 0;
-      
-      /* Find the device from the MIDI interfaces list */
-      for (i = 1; i < editor->nummidiinterfaces; i++) {
-	if (editor->midiinterfaces[i]->devicename != NULL && strcmp(editor->midiinterfaces[i]->devicename, current) == 0)
-	  break;
-      }
-
-      /* If the interface didn't exist move it to unavailable */
-      if (i >= editor->nummidiinterfaces) {
-        if (j == 0) {
-          unavailableinstring = string_append_string(unavailableinstring, current);
-          unavailableinstring = string_append_char(unavailableinstring, ';');
-        } else {
-          unavailableoutstring = string_append_string(unavailableoutstring, current);
-          unavailableoutstring = string_append_char(unavailableoutstring, ';');
-        }
-      }
-
-      current = ++next;
-    }
-  }
-
-  /* Free the old in/out strings */
-  free(instring);
-  free(outstring);
-  instring = NULL;
-  outstring = NULL;
-  
-  /* Check all MIDI interfaces for availability */
-  for (i = 1; i < editor->nummidiinterfaces; i++) {
-    direction = editor->midiinterfaces[i]->direction;
-    unavailabledirection = editor->midiinterfaces[i]->unavailabledirection;
-
-    if (direction & DIRECTION_IN) {
-      instring = string_append_string(instring, editor->midiinterfaces[i]->devicename);
-      instring = string_append_char(instring, ';');
-    }
-
-    if (direction & DIRECTION_OUT) {
-      outstring = string_append_string(outstring, editor->midiinterfaces[i]->devicename);
-      outstring = string_append_char(outstring, ';');
-    }
-
-    if (unavailabledirection & DIRECTION_IN) {
-      unavailableinstring = string_append_string(unavailableinstring, editor->midiinterfaces[i]->devicename);
-      unavailableinstring = string_append_char(unavailableinstring, ';');
-    }
-
-    if (unavailabledirection & DIRECTION_OUT) {
-      unavailableoutstring = string_append_string(unavailableoutstring, editor->midiinterfaces[i]->devicename);
-      unavailableoutstring = string_append_char(unavailableoutstring, ';');
-    }
-
-    editor->midiinterfaces[i]->unavailabledirection = 0;
-  }
-
-  /* If the unavailable string is empty use ; - otherwise it always ends with ; at this point */
-  if (unavailableinstring == NULL)
-    unavailableinstring = (char *)calloc(0, sizeof(char));
-  if (unavailableoutstring == NULL)
-    unavailableoutstring = (char *)calloc(0, sizeof(char));
-
-  if (strlen(unavailableinstring) == 0)
-    unavailableinstring = string_append_char(unavailableinstring, ';');
-  if (strlen(unavailableoutstring) == 0)
-    unavailableoutstring = string_append_char(unavailableoutstring, ';');
-
-  /* Remove all active interfaces from unavailable lists */
-  for (i = 1; i < editor->nummidiinterfaces; i++) {
-    /* Append ; to the end of the device name for searching */
-    char *devicename = string_append_char(strdup(editor->midiinterfaces[i]->devicename), ';');
-
-    direction = editor->midiinterfaces[i]->direction;
-
-    /* If input is active it must not be found from unavailable input devices */
-    if (direction & DIRECTION_IN)
-      unavailableinstring = string_remove_string(unavailableinstring, devicename);
-
-    /* If output is active it must not be found from unavailable output devices */
-    if (direction & DIRECTION_OUT)
-      unavailableoutstring = string_remove_string(unavailableoutstring, devicename);
-  }
-
-  /* If no interfaces use "", otherwise remove last separator */
-  if (instring == NULL)
-    instring = (char *)calloc(1, sizeof(char));
-  else {
-    l = strlen(instring);
-    if (l > 0) {
-      instring = realloc(instring, l);
-      instring[l - 1] = 0;
-    }
-  }
-    
-  if (outstring == NULL)
-    outstring = (char *)calloc(1, sizeof(char));
-  else {
-    l = strlen(outstring);
-    if (l > 0) {
-      outstring = realloc(outstring, l);
-      outstring[l - 1] = 0;
-    }
-  }
-
-  /* Remove last separator from unavailable strings if there is any */
-  l = strlen(unavailableinstring);
-  if (l > 0) {
-    unavailableinstring = realloc(unavailableinstring, l);
-    unavailableinstring[l - 1] = 0;
-  }
-
-  l = strlen(unavailableoutstring);
-  if (l > 0) {
-    unavailableoutstring = realloc(unavailableoutstring, l);
-    unavailableoutstring[l - 1] = 0;
-  }
-
   /* Set the MIDI interfaces */
-  editor_midi_interface_set(editor, instring, DIRECTION_IN, MIDIINTERFACES_AVAILABLE);
-  editor_midi_interface_set(editor, outstring, DIRECTION_OUT, MIDIINTERFACES_AVAILABLE);
-  editor_midi_interface_set(editor, unavailableinstring, DIRECTION_IN, MIDIINTERFACES_UNAVAILABLE);
-  editor_midi_interface_set(editor, unavailableoutstring, DIRECTION_OUT, MIDIINTERFACES_UNAVAILABLE);
-
-  free(instring);
-  free(outstring);
-  free(unavailableinstring);
-  free(unavailableoutstring);
+  char *outputs_ok = string_shorten(editor->midi->outputs_ok);
+  char *inputs_ok = string_shorten(editor->midi->inputs_ok);
+  char *outputs_error = string_shorten(editor->midi->outputs_error);
+  char *inputs_error = string_shorten(editor->midi->inputs_error);
+  editor_midi_interface_set(editor, inputs_ok, DIRECTION_IN, MIDIINTERFACES_AVAILABLE);
+  editor_midi_interface_set(editor, outputs_ok, DIRECTION_OUT, MIDIINTERFACES_AVAILABLE);
+  editor_midi_interface_set(editor, inputs_error, DIRECTION_IN, MIDIINTERFACES_UNAVAILABLE);
+  editor_midi_interface_set(editor, outputs_error, DIRECTION_OUT, MIDIINTERFACES_UNAVAILABLE);
 }
 
-/* Enables or disables a MIDI interface by name */
-unsigned int editor_midi_interface_status_set(struct editor *editor, char *devicename, unsigned int direction, unsigned int enabled)
+/* Enables or disables a MIDI interface by client and port */
+unsigned int editor_midi_interface_status_set(struct editor *editor, unsigned int client, unsigned int port, unsigned int direction, unsigned int enabled)
 {
   nullcheck_int(editor, editor_midi_interface_status_set);
-  nullcheck_int(devicename, editor_midi_interface_status_set);
-  int i, status = 0;
 
-  /* Search for the specified device, open or close as requested */
-  for (i = 1; i < editor->nummidiinterfaces; i++)
-    if (strcmp(editor->midiinterfaces[i]->devicename, devicename) == 0) {
-      if (enabled)
-	status = midi_open(editor->midiinterfaces[i], direction);
-      else
-	midi_close(editor->midiinterfaces[i], direction);
-      break;
-    }
+  unsigned int result;
 
+  if (enabled)
+    /* Subscribe */
+    result = midi_subscribe(editor->midi, client, port, direction);
+  else {
+    /* Store the old MIDI interface names */
+    editor_song_midi_interface_names_set(editor);
+
+    /* Unsubscribe */
+    result = 1 - midi_unsubscribe(editor->midi, client, port, direction);
+
+    /* Make sure the MIDI interface numbers are still valid */
+    editor_song_midi_interface_numbers_set(editor);
+  }
+
+  /* Refresh the instrument window */
+  gui_instrument_refresh(editor->gui);
+  
+  /* Update the preferences */
   editor_midi_interface_set_from_current(editor);
-  return status;
+
+  return result;
 }
 
 /* Returns the song path from the preferences */
@@ -2070,57 +1899,3 @@ void editor_player_play_note(struct editor *editor, unsigned char note, unsigned
 
   player_play_note(editor->player, editor->instrument, editor->octave * 12 + note, 127, track);
 }
-
-/* Helper functions */
-static char *string_remove_string(char *from, char *toremove)
-{
-  int fl = strlen(from) + 1;
-  int l = strlen(toremove);
-  int ml;
-
-  char *match = strstr(from, toremove);
-  while (match != NULL) {
-    ml = strlen(match);
-    memmove(match, match + l, ml - l);
-    fl -= l;
-    
-    /* Make sure the string won't completely disappear */
-    if (fl < 1)
-      fl = 1;
-    
-    from = realloc(from, fl);
-    from[fl - 1] = 0;
-    
-    match = strstr(from, toremove);
-  }
-
-  return from;
-}
-
-static char *string_append_char(char *to, char append)
-{
-  int l = 0;
-
-  if (to != NULL)
-    l = strlen(to);
-
-  to = realloc(to, l + 2);
-  to[l++] = append;
-  to[l] = 0;
-  
-  return to;
-}
-
-static char *string_append_string(char *to, char *append)
-{
-  int l1 = 0;
-  int l2 = strlen(append);
-  
-  if (to != NULL)
-    l1 = strlen(to);
-
-  to = realloc(to, l1 + l2 + 1);
-  strcpy(to + l1, append);
-
-  return to;
-}
