@@ -212,27 +212,35 @@ struct gui *gui_open(struct editor *editor)
   gtk_tree_view_append_column(treeview, new_column("Automatically Send Message After Loading Song", renderer, "active", GUI_MESSAGELIST_COLUMN_AUTOSEND, NULL));
 
   /* Preferences */
-  store = gtk_list_store_new(4, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN);
-  treeview = GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "treeview_preferences_midi_interfaces"));
+  store = gtk_list_store_new(4, G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_INT, G_TYPE_INT);
+  treeview = GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "treeview_preferences_midi_interfaces_output"));
   select = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
   gtk_tree_selection_set_mode(select, GTK_SELECTION_NONE);
   gtk_tree_view_set_model(treeview, GTK_TREE_MODEL(store));
   renderer = gtk_cell_renderer_text_new();
-  g_object_set_data(G_OBJECT(renderer), "column", GUINT_TO_POINTER(GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_DEVICENAME));
-  gtk_tree_view_append_column(treeview, new_column("Device", renderer, "text", GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_DEVICENAME, NULL));
-  renderer = gtk_cell_renderer_text_new();
   g_object_set_data(G_OBJECT(renderer), "column", GUINT_TO_POINTER(GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_NAME));
   gtk_tree_view_append_column(treeview, new_column("Name", renderer, "text", GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_NAME, NULL));
   renderer = gtk_cell_renderer_toggle_new();
   g_object_set(renderer, "activatable", TRUE, NULL);
-  g_object_set_data(G_OBJECT(renderer), "column", GUINT_TO_POINTER(GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_INPUT_ENABLED));
+  g_object_set_data(G_OBJECT(renderer), "column", GUINT_TO_POINTER(GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_ENABLED));
+  g_object_set_data(G_OBJECT(renderer), "direction", GUINT_TO_POINTER(DIRECTION_OUT));
   g_signal_connect(renderer, "toggled", G_CALLBACK(on_cell_preferences_midi_interfaces_toggled), gui);
-  gtk_tree_view_append_column(treeview, new_column("Input", renderer, "active", GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_INPUT_ENABLED, NULL));
+  gtk_tree_view_append_column(treeview, new_column("Enabled", renderer, "active", GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_ENABLED, NULL));
+
+  store = gtk_list_store_new(4, G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_INT, G_TYPE_INT);
+  treeview = GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "treeview_preferences_midi_interfaces_input"));
+  select = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
+  gtk_tree_selection_set_mode(select, GTK_SELECTION_NONE);
+  gtk_tree_view_set_model(treeview, GTK_TREE_MODEL(store));
+  renderer = gtk_cell_renderer_text_new();
+  g_object_set_data(G_OBJECT(renderer), "column", GUINT_TO_POINTER(GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_NAME));
+  gtk_tree_view_append_column(treeview, new_column("Name", renderer, "text", GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_NAME, NULL));
   renderer = gtk_cell_renderer_toggle_new();
   g_object_set(renderer, "activatable", TRUE, NULL);
-  g_object_set_data(G_OBJECT(renderer), "column", GUINT_TO_POINTER(GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_OUTPUT_ENABLED));
+  g_object_set_data(G_OBJECT(renderer), "column", GUINT_TO_POINTER(GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_ENABLED));
+  g_object_set_data(G_OBJECT(renderer), "direction", GUINT_TO_POINTER(DIRECTION_IN));
   g_signal_connect(renderer, "toggled", G_CALLBACK(on_cell_preferences_midi_interfaces_toggled), gui);
-  gtk_tree_view_append_column(treeview, new_column("Output", renderer, "active", GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_OUTPUT_ENABLED, NULL));
+  gtk_tree_view_append_column(treeview, new_column("Enabled", renderer, "active", GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_ENABLED, NULL));
 
   /* Set the keyboard octave */
   gui->editor->octave = 3;
@@ -247,13 +255,6 @@ struct gui *gui_open(struct editor *editor)
 
   /* Create the MIDI interface selectors (libglade doesn't do this if no items are defined) */
   w = glade_xml_get_widget(gui->xml, "combobox_instrumentproperties_midiinterface");
-  store = gtk_list_store_new(1, G_TYPE_STRING);
-  gtk_combo_box_set_model(GTK_COMBO_BOX(w), GTK_TREE_MODEL(store));
-  renderer = gtk_cell_renderer_text_new();
-  gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(w), renderer, TRUE);
-  gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(w), renderer, "text", 0, NULL);
-
-  w = glade_xml_get_widget(gui->xml, "combobox_receive");
   store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_INT);
   gtk_combo_box_set_model(GTK_COMBO_BOX(w), GTK_TREE_MODEL(store));
   renderer = gtk_cell_renderer_text_new();
@@ -534,8 +535,10 @@ void gui_instrument_refresh(struct gui *gui)
   GtkRange *range_hold = GTK_RANGE(glade_xml_get_widget(gui->xml, "scale_instrumentproperties_hold"));
   GtkRange *range_midich = GTK_RANGE(glade_xml_get_widget(gui->xml, "scale_instrumentproperties_midich"));
   char midiinterface[256], *name;
-  int i;
-  
+  int i, n = 0, current, selected = 0;
+
+  current = editor_song_instrument_current_midiinterface_get(gui->editor);
+
   /* Refresh the main window widgets */
   g_signal_handlers_block_by_func(spinbutton, on_spinbutton_instrument_changed, gui);
   g_signal_handlers_block_by_func(entry, on_entry_instrumentproperties_name_changed, gui);
@@ -560,18 +563,35 @@ void gui_instrument_refresh(struct gui *gui)
 
   GtkListStore *store = GTK_LIST_STORE(gtk_combo_box_get_model(combobox));
   gtk_list_store_clear(store);
-  for (i = 0; i < gui->editor->nummidiinterfaces; i++) {
+  for (i = 0; i < gui->editor->midi->numoutputs; i++) {
     GtkTreeIter iter;
 
-    if (gui->editor->midiinterfaces[i]->devicename != NULL)
-      snprintf(midiinterface, 256, "%s (%s)", gui->editor->midiinterfaces[i]->name, gui->editor->midiinterfaces[i]->devicename);
-    else
-      snprintf(midiinterface, 256, "%s", gui->editor->midiinterfaces[i]->name);
-
-    gtk_list_store_append(store, &iter);
-    gtk_list_store_set(store, &iter, 0, midiinterface, -1);
+    if (gui->editor->midi->outputs[i]->type != MIDI_ALSA) {
+      if (gui->editor->midi->outputs[i]->name != NULL)
+	snprintf(midiinterface, 256, "%s", gui->editor->midi->outputs[i]->name);
+      else
+	snprintf(midiinterface, 256, "Unknown");
+      gtk_list_store_append(store, &iter);
+      gtk_list_store_set(store, &iter, 0, midiinterface, 1, i, -1);
+      if (i == current)
+	selected = n;
+      n++;
+    } else {
+      struct midi_alsa *alsa = (struct midi_alsa *)gui->editor->midi->outputs[i]->data;
+      if (alsa->subs != NULL) {
+	if (gui->editor->midi->outputs[i]->name != NULL)
+	  snprintf(midiinterface, 256, "%s (%d:%d)", gui->editor->midi->outputs[i]->name, alsa->client, alsa->port);
+	else
+	  snprintf(midiinterface, 256, "Unknown (%d:%d)", alsa->client, alsa->port);
+	gtk_list_store_append(store, &iter);
+	gtk_list_store_set(store, &iter, 0, midiinterface, 1, i, -1);
+	if (i == current)
+	  selected = n;
+	n++;
+      }
+    }
   }
-  gtk_combo_box_set_active(combobox, editor_song_instrument_current_midiinterface_get(gui->editor));
+  gtk_combo_box_set_active(combobox, selected);
   gtk_adjustment_set_value(gtk_range_get_adjustment(range_volume), editor_song_instrument_current_defaultvelocity_get(gui->editor));
   gtk_adjustment_set_value(gtk_range_get_adjustment(range_transpose), editor_song_instrument_current_transpose_get(gui->editor));
   gtk_adjustment_set_value(gtk_range_get_adjustment(range_hold), editor_song_instrument_current_hold_get(gui->editor));
@@ -984,90 +1004,105 @@ void gui_trackvolumes_refresh(struct gui *gui)
 
 void gui_preferences_refresh(struct gui *gui)
 {
-  GtkTreeView *treeview = GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "treeview_preferences_midi_interfaces"));
+  GtkTreeView *treeview_output = GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "treeview_preferences_midi_interfaces_output"));
+  GtkTreeView *treeview_input = GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "treeview_preferences_midi_interfaces_input"));
   GtkComboBox *combobox = GTK_COMBO_BOX(glade_xml_get_widget(gui->xml, "combobox_preferences_scheduling_mode"));
   int i;
 
   /* Refresh the preferences window widgets */
-  g_signal_handlers_block_by_func(treeview, on_cell_preferences_midi_interfaces_toggled, gui);
+  g_signal_handlers_block_by_func(treeview_output, on_cell_preferences_midi_interfaces_toggled, gui);
+  g_signal_handlers_block_by_func(treeview_input, on_cell_preferences_midi_interfaces_toggled, gui);
   g_signal_handlers_block_by_func(combobox, on_combobox_preferences_scheduling_mode_changed, gui);
 
-  /* Recreate the Message list */
-  GtkListStore *store = GTK_LIST_STORE(gtk_tree_view_get_model(treeview));
+  /* Recreate the interfaces lists */
+  GtkListStore *store = GTK_LIST_STORE(gtk_tree_view_get_model(treeview_output));
   gtk_list_store_clear(store);
-  for (i = 1; i < gui->editor->nummidiinterfaces; i++) {
+  for (i = 1; i < gui->editor->midi->numoutputs; i++) {
     GtkTreeIter iter;
-    char *device, *name;
-    gboolean inputenabled = FALSE, outputenabled = FALSE;
+    char name[256];
+    gboolean enabled = FALSE;
+    int client = 0, port = 0;
 
-    if (gui->editor->midiinterfaces[i]->devicename != NULL)
-      device = gui->editor->midiinterfaces[i]->devicename;
-    else
-      device = "";
-    if (gui->editor->midiinterfaces[i]->name != NULL)
-      name = gui->editor->midiinterfaces[i]->name;
-    else
-      name = "";
-    if (gui->editor->midiinterfaces[i]->direction & DIRECTION_IN)
-      inputenabled = TRUE;
-    if (gui->editor->midiinterfaces[i]->direction & DIRECTION_OUT)
-      outputenabled = TRUE;
+    if (gui->editor->midi->outputs[i]->type == MIDI_ALSA) {
+      struct midi_alsa *alsa = (struct midi_alsa *)gui->editor->midi->outputs[i]->data;
+
+      if (alsa->subs != NULL)
+	enabled = TRUE;
+
+      client = alsa->client;
+      port = alsa->port;
+
+      if (gui->editor->midi->outputs[i]->name != NULL)
+	snprintf(name, 256, "%s (%d:%d)", gui->editor->midi->outputs[i]->name, client, port);
+      else
+	snprintf(name, 256, "Unknown (%d:%d)", client, port);
+    } else {
+      if (gui->editor->midi->outputs[i]->name != NULL)
+	snprintf(name, 256, "%s", gui->editor->midi->outputs[i]->name);
+      else
+	snprintf(name, 256, "Unknown");
+    }
 
     gtk_list_store_append(store, &iter);
-    gtk_list_store_set(store, &iter, GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_DEVICENAME, device, GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_NAME, name, GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_INPUT_ENABLED, inputenabled, GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_OUTPUT_ENABLED, outputenabled, -1);
+    gtk_list_store_set(store, &iter, GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_NAME, name, GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_ENABLED, enabled, GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_CLIENT, client, GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_PORT, port, -1);
+  }
+
+  store = GTK_LIST_STORE(gtk_tree_view_get_model(treeview_input));
+  gtk_list_store_clear(store);
+  for (i = 0; i < gui->editor->midi->numinputs; i++) {
+    GtkTreeIter iter;
+    char name[256];
+    gboolean enabled = FALSE;
+    int client = 0, port = 0;
+
+    if (gui->editor->midi->inputs[i]->type == MIDI_ALSA) {
+      struct midi_alsa *alsa = (struct midi_alsa *)gui->editor->midi->inputs[i]->data;
+
+      if (alsa->subs != NULL)
+	enabled = TRUE;
+
+      client = alsa->client;
+      port = alsa->port;
+
+      if (gui->editor->midi->inputs[i]->name != NULL)
+	snprintf(name, 256, "%s (%d:%d)", gui->editor->midi->inputs[i]->name, client, port);
+      else
+	snprintf(name, 256, "Unknown (%d:%d)", client, port);
+    } else {
+      if (gui->editor->midi->inputs[i]->name != NULL)
+	snprintf(name, 256, "%s", gui->editor->midi->inputs[i]->name);
+      else
+	snprintf(name, 256, "Unknown");
+    }
+
+    gtk_list_store_append(store, &iter);
+    gtk_list_store_set(store, &iter, GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_NAME, name, GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_ENABLED, enabled, GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_CLIENT, client, GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_PORT, port, -1);
   }
 
   gtk_combo_box_set_active(combobox, editor_scheduler_get(gui->editor) - SCHED_RTC);
-  g_signal_handlers_unblock_by_func(treeview, on_cell_preferences_midi_interfaces_toggled, gui);
+  g_signal_handlers_unblock_by_func(treeview_output, on_cell_preferences_midi_interfaces_toggled, gui);
+  g_signal_handlers_unblock_by_func(treeview_input, on_cell_preferences_midi_interfaces_toggled, gui);
   g_signal_handlers_unblock_by_func(combobox, on_combobox_preferences_scheduling_mode_changed, gui);
 }
 
-void gui_receive_refresh(struct gui *gui, unsigned int bytes, unsigned int total, gboolean recreate)
+void gui_receive_refresh(struct gui *gui, int bytes, int total)
 {
-  GtkComboBox *combobox = GTK_COMBO_BOX(glade_xml_get_widget(gui->xml, "combobox_receive"));
-  GtkListStore *store = GTK_LIST_STORE(gtk_combo_box_get_model(combobox));
   GtkProgressBar *pbar = GTK_PROGRESS_BAR(glade_xml_get_widget(gui->xml, "progressbar_receive"));
-  int i, n = 0;
-  char b[40], midiinterface[256];
-
-  /* Recreate the MIDI interface combo box */
-  if (recreate) {
-    gtk_list_store_clear(store);
-    for (i = 1; i < gui->editor->nummidiinterfaces; i++) {
-      /* Add all inbound MIDI interfaces */
-      if (gui->editor->midiinterfaces[i]->direction & DIRECTION_IN) {
-	GtkTreeIter iter;
-
-	if (gui->editor->midiinterfaces[i]->devicename != NULL)
-	  snprintf(midiinterface, 256, "%s (%s)", gui->editor->midiinterfaces[i]->name, gui->editor->midiinterfaces[i]->devicename);
-	else
-	  snprintf(midiinterface, 256, "%s", gui->editor->midiinterfaces[i]->name);
-
-	gtk_list_store_append(store, &iter);
-	gtk_list_store_set(store, &iter, 0, midiinterface, 1, i, -1);
-	n++;
-      }
-    }
-
-    /* Receiving is only possible if at least one inbound MIDI interface found */
-    if (n > 0) {
-      gtk_combo_box_set_active(combobox, 0);
-      gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_receive"), TRUE);
-    } else {
-      gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_receive"), FALSE);
-      gui->editor->receivemidiinterface = -1;
-    }
-  }
+  char b[40];
 
   /* Update bytes received count: if total is 0 it is unknown and not shown */
   if (total == 0) {
-    snprintf(b, 40, "Bytes received: %d", bytes);
-    if (!recreate)
+    if (bytes >= 0) {
+      snprintf(b, 40, "Bytes received: %d", bytes);
       gtk_progress_bar_pulse(pbar);
+    } else
+      snprintf(b, 40, "Bytes received: 0");
   } else {
-    snprintf(b, 40, "Bytes received: %d/%d", bytes, total);
-    if (!recreate)
+    if (bytes >= 0) {
+      snprintf(b, 40, "Bytes received: %d/%d", bytes, total);
       gtk_progress_bar_set_fraction(pbar, (gdouble) bytes / total);
+    } else
+      snprintf(b, 40, "Bytes received: 0/0");
   }
 
   gtk_progress_bar_set_text(pbar, b);
