@@ -26,7 +26,6 @@
 #include <time.h>
 #include <sys/time.h>
 #include <sys/ioctl.h>
-#include <unistd.h>
 #include <fcntl.h>
 #include <glib.h>
 #include <sched.h>
@@ -94,10 +93,10 @@ void player_play_note(struct player *player, unsigned int instrument, unsigned c
   struct song *song = player->song;
 
   /* Stop currently playing note */
-  for (i = 0; i < player->nummidiinterfaces; i++) {
+  for (i = 0; i < player->trackstatus[track]->numinterfaces; i++) {
     struct player_interface_status *status = player->trackstatus[track]->interfaces[i];
     if (status->note != -1) {
-      midi_note_off(player->midiinterfaces[i], status->midichannel, status->note, 127);
+      midi_note_off(player->midi->outputs[i], status->midichannel, status->note, 127);
       status->note = -1;
     }
   }
@@ -119,9 +118,9 @@ void player_play_note(struct player *player, unsigned int instrument, unsigned c
       /* If channels are soloed solo must be taken into account, otherwise not */
       /* NOTE: Current behaviour does NOT ignore muted or non-soloed channels. The channels are played with volume 0 causing note offs. This is by design. Is it relevant? */
       if (player->solo)
-        status->volume = output->defaultvelocity * volume / 127 * song->tracks[track]->volume / 127 * song->tracks[track]->solo * (1 - song->tracks[track]->mute) * song->mastervolume / 127;
+	status->volume = output->defaultvelocity * volume / 127 * song->tracks[track]->volume / 127 * song->tracks[track]->solo * (1 - song->tracks[track]->mute) * song->mastervolume / 127;
       else
-        status->volume = output->defaultvelocity * volume / 127 * song->tracks[track]->volume / 127 * (1 - song->tracks[track]->mute) * song->mastervolume / 127;        
+	status->volume = output->defaultvelocity * volume / 127 * song->tracks[track]->volume / 127 * (1 - song->tracks[track]->mute) * song->mastervolume / 127;
       if (output->hold > 0)
 	status->hold = output->hold;
       else
@@ -132,7 +131,7 @@ void player_play_note(struct player *player, unsigned int instrument, unsigned c
 	status->volume = 127;
 
       /* Play note */
-      midi_note_on(player->midiinterfaces[output->midiinterface], status->midichannel, status->note, status->volume);
+      midi_note_on(player->midi->outputs[output->midiinterface], status->midichannel, status->note, status->volume);
     }
   }
 }
@@ -145,11 +144,11 @@ void player_stop_notes(struct player *player)
   int i, j;
 
   for (i = 0; i < player->song->maxtracks; i++) {
-    for (j = 0; j < player->nummidiinterfaces; j++) {
+    for (j = 0; j < player->trackstatus[i]->numinterfaces; j++) {
       if (player->trackstatus[i]->interfaces[j]->note != -1) {
 	struct player_interface_status *status = player->trackstatus[i]->interfaces[j];
 
-	midi_note_off(player->midiinterfaces[j], status->midichannel, status->note, 127);
+	midi_note_off(player->midi->outputs[j], status->midichannel, status->note, 127);
 
 	status->midichannel = -1;
 	status->note = -1;
@@ -169,8 +168,8 @@ void player_stop_all_notes(struct player *player)
 
   for (i = 0; i < 16; i++)
     for (j = 0; j < 128; j++)
-      for (k = 0; k < player->nummidiinterfaces; k++)
-	midi_note_off(player->midiinterfaces[k], i, j, 127);
+      for (k = 0; k < player->midi->numoutputs; k++)
+	midi_note_off(player->midi->outputs[k], i, j, 127);
 }
 
 /* Resets the pitch wheel on all channels */
@@ -181,8 +180,8 @@ void player_reset_pitch(struct player *player)
   int i, j;
 
   for (i = 0; i < 16; i++)
-    for (j = 0; j < player->nummidiinterfaces; j++)
-      midi_pitch_wheel(player->midiinterfaces[j], i, 64);
+    for (j = 0; j < player->midi->numoutputs; j++)
+      midi_pitch_wheel(player->midi->outputs[j], i, 64);
 }
 
 /* Player thread */
@@ -249,9 +248,13 @@ static void player_thread(struct player *player)
       }
     }
 
+    /* Lock */
+    if (player->mutex != NULL)
+      g_mutex_lock(player->mutex);
+
     /* Handle this tick */
-    for (i = 0; i < player->nummidiinterfaces; i++)
-      midi_set_tick(player->midiinterfaces[i], player->tickssofar);
+    for (i = 0; i < player->midi->numoutputs; i++)
+      midi_set_tick(player->midi->outputs[i], player->tickssofar);
 
     if (player->editor != NULL)
       editor_song_lock(player->editor);
@@ -260,8 +263,8 @@ static void player_thread(struct player *player)
 
     /* Send MIDI sync if requested */
     if (song->sendsync)
-      for (i = 0; i < player->nummidiinterfaces; i++)
-	midi_clock(player->midiinterfaces[i]);
+      for (i = 0; i < player->midi->numoutputs; i++)
+	midi_clock(player->midi->outputs[i]);
 
     /* Pass 1: Stop notes if there are new notes about to be played */
     for (i = 0; i < b->tracks; i++) {
@@ -295,10 +298,10 @@ static void player_thread(struct player *player)
 
 	/* Stop currently playing note */
 	if ((delay >= 0 && player->tick == delay) || (delay < 0 && player->tick % (-delay) == 0)) {
-	  for (j = 0; j < player->nummidiinterfaces; j++) {
+	  for (j = 0; j < trackstatus->numinterfaces; j++) {
 	    struct player_interface_status *interfacestatus = trackstatus->interfaces[j];
 	    if (interfacestatus->note != -1) {
-	      midi_note_off(player->midiinterfaces[j], interfacestatus->midichannel, interfacestatus->note, 127);
+	      midi_note_off(player->midi->outputs[j], interfacestatus->midichannel, interfacestatus->note, 127);
 	      interfacestatus->note = -1;
 	    }
 	  }
@@ -334,7 +337,7 @@ static void player_thread(struct player *player)
 	    }
 	  } else {
 	    /* Note playing defines MIDI interfaces/channels */
-	    numoutputs = player->nummidiinterfaces;
+	    numoutputs = player->trackstatus[i]->numinterfaces;
 	    outputs = (unsigned int *)calloc(numoutputs * 2, sizeof(unsigned int));
 	    for (k = 0; k < numoutputs; k++) {
 	      outputs[k * 2 + 0] = k;
@@ -346,7 +349,7 @@ static void player_thread(struct player *player)
 	  for (k = 0; k < numoutputs; k++) {
 	    int midiinterface = outputs[k * 2 + 0];
 	    int midichannel = outputs[k * 2 + 1];
-	    struct midi *midi = player->midiinterfaces[midiinterface];
+	    struct midi_interface *midi = player->midi->outputs[midiinterface];
 	    struct player_interface_status *interfacestatus = trackstatus->interfaces[midiinterface];
 
 	    /* Check for previous command if any */
@@ -480,7 +483,7 @@ static void player_thread(struct player *player)
 		  postcommand = COMMAND_TEMPO;
 	      } else {
 		song_set_tempo(song, value);
-                midi_tempo(midi, value);
+		midi_tempo(midi, value);
 		changetempo = 1;
 	      }
 	      break;
@@ -533,11 +536,11 @@ static void player_thread(struct player *player)
 	  for (j = 0; j < instr->numoutputs; j++) {
 	    struct instrument_output *output = instr->outputs[j];
 	    struct player_interface_status *interfacestatus = trackstatus->interfaces[output->midiinterface];
-	    
+
 	    /* If no hold value was defined use the instrument's hold value */
 	    if (hold == -1)
 	      hold = output->hold;
-	    
+
 	    if (hold == 0)
 	      interfacestatus->hold = -1;
 	    else
@@ -547,7 +550,7 @@ static void player_thread(struct player *player)
       }
 
       /* First tick, no note but instrument defined? */
-      for (j = 0; j < player->nummidiinterfaces; j++) {
+      for (j = 0; j < trackstatus->numinterfaces; j++) {
 	struct player_interface_status *interfacestatus = trackstatus->interfaces[j];
 	if (player->tick == 0 && note == 0 && instrument > 0 && interfacestatus->hold >= 0)
 	  for (k = 0; k < song->instruments[instrument - 1]->numoutputs; k++)
@@ -559,12 +562,12 @@ static void player_thread(struct player *player)
     /* Decrement hold times of notes and stop notes that should be stopped */
     for (i = 0; i < song->maxtracks; i++) {
       struct player_track_status *trackstatus = player->trackstatus[i];
-      for (j = 0; j < player->nummidiinterfaces; j++) {
+      for (j = 0; j < trackstatus->numinterfaces; j++) {
 	struct player_interface_status *interfacestatus = trackstatus->interfaces[j];
 	if (interfacestatus->hold >= 0) {
 	  interfacestatus->hold--;
 	  if (interfacestatus->hold < 0 && interfacestatus->note != -1) {
-	    midi_note_off(player->midiinterfaces[j], interfacestatus->midichannel, interfacestatus->note, 127);
+	    midi_note_off(player->midi->outputs[j], interfacestatus->midichannel, interfacestatus->note, 127);
 	    interfacestatus->note = -1;
 	  }
 	}
@@ -601,11 +604,7 @@ static void player_thread(struct player *player)
 	break;
       case COMMAND_TEMPO:
 	/* COMMAND_TPL and COMMAND_TEMPO can only mean "stop" as stop cmds */
-	if (player->mutex != NULL)
-	  g_mutex_lock(player->mutex);
 	player->killthread = 1;
-	if (player->mutex != NULL)
-	  g_mutex_unlock(player->mutex);
 	return;
 	break;
       default:
@@ -643,8 +642,6 @@ static void player_thread(struct player *player)
       editor_song_unlock(player->editor);
 
     /* Check whether this thread should be killed */
-    if (player->mutex != NULL)
-      g_mutex_lock(player->mutex);
     if (player->killthread || (player->sched == SCHED_NONE && looped))
       break;
     if (player->mutex != NULL)
@@ -671,16 +668,8 @@ static void player_thread(struct player *player)
     if (player->sched == SCHED_RTC)
       ioctl(player->rtc, RTC_PIE_OFF, 0);
 
-    /* This is here because sometimes the player didn't stop all notes. Reason:
-     * unknown. Seemed like a flush bug somewhere. */
-    usleep(25000);
-
     /* Stop all playing notes */
     player_stop_notes(player);
-
-    /* This is here because sometimes the player didn't stop all notes. Reason:
-     * unknown. Seemed like a flush bug somewhere. */
-    usleep(25000);
   }
 
   /* The mutex is locked if the thread was killed and loop broken */
@@ -693,7 +682,7 @@ static void player_thread(struct player *player)
 }
 
 /* Plays a song to a MIDI device witout any scheduling (for export) */
-void player_midi_export(struct song *song, struct midi *midi)
+void player_midi_export(struct song *song, struct midi_interface *midi)
 {
   int i, j;
   struct player *player = (struct player *)calloc(1, sizeof(struct player));
@@ -703,16 +692,16 @@ void player_midi_export(struct song *song, struct midi *midi)
     struct instrument *instrument = song->instruments[i];
     for (j = 0; j < instrument->numoutputs; j++) {
       struct instrument_output *output = instrument->outputs[j];
-      if (output->midiinterface > player->nummidiinterfaces)
-	player->nummidiinterfaces = output->midiinterface;
+      if (output->midiinterface > player->midi->numoutputs)
+	player->midi->numoutputs = output->midiinterface;
     }
   }
-  player->nummidiinterfaces++;
+  player->midi->numoutputs++;
 
   player->song = song;
-  player->midiinterfaces = (struct midi **)calloc(player->nummidiinterfaces, sizeof(struct midi *));
-  for (i = 0; i < player->nummidiinterfaces; i++)
-    player->midiinterfaces[i] = midi;
+  player->midi->outputs = (struct midi_interface **)calloc(player->midi->numoutputs, sizeof(struct midi_interface *));
+  for (i = 0; i < player->midi->numoutputs; i++)
+    player->midi->outputs[i] = midi;
   player->sched = SCHED_NONE;
   player->mode = MODE_PLAY_SONG;
   player_refresh_playseq_and_block(player);
@@ -721,26 +710,26 @@ void player_midi_export(struct song *song, struct midi *midi)
   player_trackstatus_create(player, 0);
 
   /* Create MIDI controller values array */
-  player->midicontrollervalues = (unsigned char **)calloc(player->nummidiinterfaces, sizeof(unsigned char *));
-  for (i = 0; i < player->nummidiinterfaces; i++)
+  player->midicontrollervalues = (unsigned char **)calloc(player->midi->numoutputs, sizeof(unsigned char *));
+  for (i = 0; i < player->midi->numoutputs; i++)
     player->midicontrollervalues[i] = (unsigned char *)calloc(16 * VALUES, sizeof(unsigned char));
 
   /* Send messages to be autosent */
   for (i = 0; i < player->song->nummessages; i++) {
     if (player->song->messages[i]->autosend)
-      midi_write_raw(player->midiinterfaces[0], player->song->messages[i]->data, player->song->messages[i]->length);
+      midi_write_raw(player->midi->outputs[0], player->song->messages[i]->data, player->song->messages[i]->length);
   }
 
   /* Set tempo */
-  midi_tempo(player->midiinterfaces[0], player->song->tempo);
-  
+  midi_tempo(player->midi->outputs[0], player->song->tempo);
+
   player_thread(player);
   player_stop_notes(player);
 
   /* Free the track status array */
   if (player->trackstatus != NULL) {
     for (i = 0; i < player->song->maxtracks; i++) {
-      for (j = 0; j < player->nummidiinterfaces; j++)
+      for (j = 0; j < player->trackstatus[i]->numinterfaces; j++)
 	free(player->trackstatus[i]->interfaces[j]);
       free(player->trackstatus[i]->interfaces);
       free(player->trackstatus[i]);
@@ -749,20 +738,19 @@ void player_midi_export(struct song *song, struct midi *midi)
   }
 
   /* Free MIDI interface array */
-  free(player->midiinterfaces);
+  free(player->midi->outputs);
   free(player);
 }
 
 /* Creates a player for a song */
-struct player *player_open(struct song *song, struct editor *editor, struct midi **midiinterfaces, unsigned int nummidiinterfaces, unsigned int scheduler)
+struct player *player_open(struct song *song, struct editor *editor, struct midi *midi, unsigned int scheduler)
 {
   struct player *player = (struct player *)calloc(1, sizeof(struct player));
   int i, j;
 
   player->song = song;
   player->editor = editor;
-  player->midiinterfaces = midiinterfaces;
-  player->nummidiinterfaces = nummidiinterfaces;
+  player->midi = midi;
   player->sched = scheduler;
   if (player->sched == SCHED_RTC) {
     player->rtc = open("/dev/rtc", O_RDONLY);
@@ -778,18 +766,19 @@ struct player *player_open(struct song *song, struct editor *editor, struct midi
   player_trackstatus_create(player, 0);
 
   /* Create MIDI controller values array */
-  player->midicontrollervalues = (unsigned char **)calloc(player->nummidiinterfaces, sizeof(unsigned char *));
-  for (i = 0; i < player->nummidiinterfaces; i++)
+  player->nummidicontrollervalues = player->midi->numoutputs;
+  player->midicontrollervalues = (unsigned char **)calloc(player->nummidicontrollervalues, sizeof(unsigned char *));
+  for (i = 0; i < player->nummidicontrollervalues; i++)
     player->midicontrollervalues[i] = (unsigned char *)calloc(16 * VALUES, sizeof(unsigned char));
 
   /* Check solo status */
   player_check_solo(player);
-  
+
   /* Send messages to be autosent */
   for (i = 0; i < player->song->nummessages; i++) {
     if (player->song->messages[i]->autosend)
-      for (j = 0; j < player->nummidiinterfaces; j++)
-	midi_write_raw(player->midiinterfaces[j], player->song->messages[i]->data, player->song->messages[i]->length);
+      for (j = 0; j < player->midi->numoutputs; j++)
+	midi_write_raw(player->midi->outputs[j], player->song->messages[i]->data, player->song->messages[i]->length);
   }
 
   return player;
@@ -811,7 +800,7 @@ void player_close(struct player *player)
 
   /* Free MIDI controller values array */
   if (player->midicontrollervalues) {
-    for (i = 0; i < player->nummidiinterfaces; i++)
+    for (i = 0; i < player->nummidicontrollervalues; i++)
       free(player->midicontrollervalues[i]);
     free(player->midicontrollervalues);
   }
@@ -819,7 +808,7 @@ void player_close(struct player *player)
   /* Free the track status array */
   if (player->trackstatus != NULL) {
     for (i = 0; i < player->song->maxtracks; i++) {
-      for (j = 0; j < player->nummidiinterfaces; j++)
+      for (j = 0; j < player->trackstatus[i]->numinterfaces; j++)
 	free(player->trackstatus[i]->interfaces[j]);
       free(player->trackstatus[i]->interfaces);
       free(player->trackstatus[i]);
@@ -905,9 +894,9 @@ void player_trackstatus_create(struct player *player, int oldmax)
   int i, j;
 
   /* Free the extraneous status structures */
-  if (player->song->maxtracks > oldmax)
+  if (player->song->maxtracks < oldmax)
     for (i = player->song->maxtracks; i < oldmax; i++) {
-      for (j = 0; j < player->nummidiinterfaces; j++)
+      for (j = 0; j < player->trackstatus[i]->numinterfaces; j++)
 	free(player->trackstatus[i]->interfaces[j]);
       free(player->trackstatus[i]->interfaces);
       free(player->trackstatus[i]);
@@ -918,10 +907,11 @@ void player_trackstatus_create(struct player *player, int oldmax)
   /* Set new tracks to -1 */
   for (i = oldmax; i < player->song->maxtracks; i++) {
     player->trackstatus[i] = (struct player_track_status *)calloc(1, sizeof(struct player_track_status));
-    player->trackstatus[i]->interfaces = (struct player_interface_status **)calloc(player->nummidiinterfaces, sizeof(struct player_interface_status *));
+    player->trackstatus[i]->numinterfaces = player->midi->numoutputs;
+    player->trackstatus[i]->interfaces = (struct player_interface_status **)calloc(player->midi->numoutputs, sizeof(struct player_interface_status *));
     player->trackstatus[i]->instrument = -1;
     player->trackstatus[i]->previouscommand = 0;
-    for (j = 0; j < player->nummidiinterfaces; j++) {
+    for (j = 0; j < player->trackstatus[i]->numinterfaces; j++) {
       player->trackstatus[i]->interfaces[j] = (struct player_interface_status *)calloc(1, sizeof(struct player_interface_status));
       player->trackstatus[i]->interfaces[j]->note = -1;
       player->trackstatus[i]->interfaces[j]->midichannel = -1;
@@ -1027,7 +1017,7 @@ void player_reset_time(struct player *player, gboolean reset_sofar)
 void player_set_solo(struct player *player, unsigned int solo)
 {
   nullcheck_void(player, player_set_solo);
-  
+
   player->solo = solo;
 }
 
@@ -1035,11 +1025,71 @@ void player_set_solo(struct player *player, unsigned int solo)
 void player_check_solo(struct player *player)
 {
   nullcheck_void(player, player_check_solo);
-  
+
   int i, solo = 0;
-  
+
   for (i = 0; i < player->song->maxtracks; i++)
     solo |= player->song->tracks[i]->solo;
 
   player->solo = solo;
 }
+
+/* Notifies the player that MIDI interfaces have changed */
+void player_midi_changed(struct player *player)
+{
+  nullcheck_void(player, player_midi_changed);
+
+  int i, j;
+
+  /* Recreate the track status array */
+  if (player->trackstatus != NULL) {
+    for (i = 0; i < player->song->maxtracks; i++) {
+      /* Remove extraneous interfaces */
+      if (player->midi->numoutputs < player->trackstatus[i]->numinterfaces)
+	for (j = player->midi->numoutputs; j < player->trackstatus[i]->numinterfaces; j++)
+	  free(player->trackstatus[i]->interfaces[j]);
+
+      player->trackstatus[i]->interfaces = (struct player_interface_status **)realloc(player->trackstatus[i]->interfaces, player->midi->numoutputs * sizeof(struct player_interface_status *));
+
+      /* Create new interfaces */
+      for (j = player->trackstatus[i]->numinterfaces; j < player->midi->numoutputs; j++) {
+	player->trackstatus[i]->interfaces[j] = (struct player_interface_status *)calloc(1, sizeof(struct player_interface_status));
+	player->trackstatus[i]->interfaces[j]->note = -1;
+	player->trackstatus[i]->interfaces[j]->midichannel = -1;
+	player->trackstatus[i]->interfaces[j]->volume = -1;
+	player->trackstatus[i]->interfaces[j]->hold = -1;
+      }
+
+      player->trackstatus[i]->numinterfaces = player->midi->numoutputs;
+    }
+  }
+
+  /* Recreate MIDI controller values array */
+  /* Remove extraneous controller values */
+  if (player->midi->numoutputs < player->nummidicontrollervalues)
+    for (i = player->midi->numoutputs; i < player->nummidicontrollervalues; i++)
+      free(player->midicontrollervalues[i]);
+
+  player->midicontrollervalues = (unsigned char **)realloc(player->midicontrollervalues, player->midi->numoutputs * sizeof(unsigned char *));
+
+  /* Create new controller values */
+  for (i = player->nummidicontrollervalues; i < player->midi->numoutputs; i++)
+    player->midicontrollervalues[i] = (unsigned char *)calloc(16 * VALUES, sizeof(unsigned char));
+  player->nummidicontrollervalues = player->midi->numoutputs;
+}
+
+/* Lock the player */
+void player_lock(struct player *player) {
+  nullcheck_void(player, player_lock);
+
+  if (player->mutex != NULL)
+    g_mutex_lock(player->mutex);
+}
+
+/* Unlock the player */
+void player_unlock(struct player *player) {
+  nullcheck_void(player, player_unlock);
+
+  if (player->mutex != NULL)
+    g_mutex_unlock(player->mutex);
+}
