@@ -1,6 +1,6 @@
 /* song.c
  *
- * Copyright 2002-2004 Vesa Halttunen
+ * Copyright 2002-2005 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -22,6 +22,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <libxml/tree.h>
+#include "tutka.h"
 #include "song.h"
 
 /* Allocates a new song structure and initializes it to default values */
@@ -42,14 +43,17 @@ struct song *song_alloc(void)
   song->numblocks = 1;
   song->blocks = (struct block **)calloc(1, sizeof(struct block *));
   song->blocks[0] = block_alloc(4, 64, 1);
-  song->numinstruments = 1;
-  song->instruments = (struct instrument **)calloc(1, sizeof(struct instrument *));
-  song->instruments[0] = instrument_alloc();
   song->maxtracks = 4;
-  song->trackvolumes = (unsigned int *)calloc(4, sizeof(unsigned int));
+  song->tracks = (struct track **)calloc(song->maxtracks, sizeof(struct track *));
   song->mastervolume = 127;
-  for (i = 0; i < song->maxtracks; i++)
-    song->trackvolumes[i] = 127;
+  for (i = 0; i < song->maxtracks; i++) {
+    /* Give a descriptive name for the new track */
+    char name[10];
+    snprintf(name, 10, "Track %d", i + 1);
+    song->tracks[i] = (struct track *)calloc(1, sizeof(struct track));
+    song->tracks[i]->volume = 127;
+    song->tracks[i]->name = strdup(name);
+  }
 
   return (song);
 }
@@ -57,10 +61,9 @@ struct song *song_alloc(void)
 /* Frees a song structure and its contents */
 void song_free(struct song *song)
 {
-  if (song == NULL) {
-    fprintf(stderr, "song_free() called with null song\n");
-    return;
-  }
+  nullcheck_void(song, song_free);
+
+  int i;
 
   if (song->name)
     free(song->name);
@@ -69,8 +72,6 @@ void song_free(struct song *song)
     free(song->sections);
 
   if (song->numplayseqs > 0) {
-    int i;
-
     for (i = 0; i < song->numplayseqs; i++)
       playseq_free(song->playseqs[i]);
 
@@ -78,8 +79,6 @@ void song_free(struct song *song)
   }
 
   if (song->numblocks > 0) {
-    int i;
-
     for (i = 0; i < song->numblocks; i++)
       block_free(song->blocks[i]);
 
@@ -87,8 +86,6 @@ void song_free(struct song *song)
   }
 
   if (song->numinstruments > 0) {
-    int i;
-
     for (i = 0; i < song->numinstruments; i++)
       if (song->instruments[i] != NULL)
 	instrument_free(song->instruments[i]);
@@ -96,8 +93,11 @@ void song_free(struct song *song)
     free(song->instruments);
   }
 
-  if (song->maxtracks > 0)
-    free(song->trackvolumes);
+  if (song->maxtracks > 0) {
+    for (i = 0; i < song->maxtracks; i++)
+      free(song->tracks[i]);
+    free(song->tracks);
+  }
 
   free(song);
 }
@@ -105,12 +105,9 @@ void song_free(struct song *song)
 /* Inserts a new block in the block array in the given position */
 void song_insert_block(struct song *song, int pos, int current)
 {
-  int i;
+  nullcheck_void(song, song_insert_block);
 
-  if (song == NULL) {
-    fprintf(stderr, "song_insert_block() called with null song\n");
-    return;
-  }
+  int i;
 
   /* Check block existence */
   if (pos < 0)
@@ -142,12 +139,9 @@ void song_insert_block(struct song *song, int pos, int current)
 /* Deletes a block from the given position of the block array */
 void song_delete_block(struct song *song, int pos)
 {
-  int i;
+  nullcheck_void(song, song_delete_block);
 
-  if (song == NULL) {
-    fprintf(stderr, "song_delete_block() called with null song\n");
-    return;
-  }
+  int i;
 
   /* Check block existence */
   if (pos < 0)
@@ -182,12 +176,9 @@ void song_delete_block(struct song *song, int pos)
 /* Inserts a new playseq in the playseq array in the given position */
 void song_insert_playseq(struct song *song, int pos)
 {
-  int i;
+  nullcheck_void(song, song_insert_playseq);
 
-  if (song == NULL) {
-    fprintf(stderr, "song_insert_playseq() called with null song\n");
-    return;
-  }
+  int i;
 
   /* Check playseq existence */
   if (pos < 0)
@@ -216,12 +207,9 @@ void song_insert_playseq(struct song *song, int pos)
 /* Deletes a playseq from the given position of the playseq array */
 void song_delete_playseq(struct song *song, int pos)
 {
-  int i;
+  nullcheck_void(song, song_delete_playseq);
 
-  if (song == NULL) {
-    fprintf(stderr, "song_delete_playseq() called with null song\n");
-    return;
-  }
+  int i;
 
   /* Check block existence */
   if (pos < 0)
@@ -254,14 +242,11 @@ void song_delete_playseq(struct song *song, int pos)
 /* Inserts a new section in the section array in the given position */
 void song_insert_section(struct song *song, int pos)
 {
+  nullcheck_void(song, song_insert_section);
+
   int i;
   unsigned int old;
 
-  if (song == NULL) {
-    fprintf(stderr, "song_insert_section() called with null song\n");
-    return;
-  }
-
   /* Check that the value is possible */
   if (pos < 0)
     pos = 0;
@@ -288,12 +273,9 @@ void song_insert_section(struct song *song, int pos)
 /* Deletes a section from the given position of the section array */
 void song_delete_section(struct song *song, int pos)
 {
-  int i;
+  nullcheck_void(song, song_delete_section);
 
-  if (song == NULL) {
-    fprintf(stderr, "song_delete_section() called with null song\n");
-    return;
-  }
+  int i;
 
   /* Check block existence */
   if (pos < 0)
@@ -317,12 +299,9 @@ void song_delete_section(struct song *song, int pos)
 /* Inserts a new message in the message array in the given position */
 void song_insert_message(struct song *song, int pos)
 {
-  int i;
+  nullcheck_void(song, song_insert_message);
 
-  if (song == NULL) {
-    fprintf(stderr, "song_insert_message() called with null song\n");
-    return;
-  }
+  int i;
 
   /* Check message existence */
   if (pos < 0)
@@ -345,12 +324,9 @@ void song_insert_message(struct song *song, int pos)
 /* Deletes a message from the given position of the message array */
 void song_delete_message(struct song *song, int pos)
 {
-  int i;
+  nullcheck_void(song, song_delete_message);
 
-  if (song == NULL) {
-    fprintf(stderr, "song_delete_message() called with null song\n");
-    return;
-  }
+  int i;
 
   /* Check message existence */
   if (pos < 0)
@@ -380,10 +356,7 @@ void song_delete_message(struct song *song, int pos)
 /* Sets a section in the given position to point somewhere */
 void song_set_section(struct song *song, int pos, int playseq)
 {
-  if (song == NULL) {
-    fprintf(stderr, "song_set_section() called with null song\n");
-    return;
-  }
+  nullcheck_void(song, song_set_section);
 
   if (playseq >= 0 && playseq < song->numplayseqs)
     song->sections[pos] = playseq;
@@ -392,10 +365,7 @@ void song_set_section(struct song *song, int pos, int playseq)
 /* Sets the number of ticks per line for a song */
 void song_set_tpl(struct song *song, unsigned int ticksperline)
 {
-  if (song == NULL) {
-    fprintf(stderr, "song_set_tpl() called with null song\n");
-    return;
-  }
+  nullcheck_void(song, song_set_tpl);
 
   song->ticksperline = ticksperline;
 }
@@ -403,10 +373,7 @@ void song_set_tpl(struct song *song, unsigned int ticksperline)
 /* Sets the tempo of a song */
 void song_set_tempo(struct song *song, unsigned int tempo)
 {
-  if (song == NULL) {
-    fprintf(stderr, "song_set_tempo() called with null song\n");
-    return;
-  }
+  nullcheck_void(song, song_set_tempo);
 
   song->tempo = tempo;
 }
@@ -414,12 +381,9 @@ void song_set_tempo(struct song *song, unsigned int tempo)
 /* If the maximum number of tracks has changed recreate the track volumes */
 int song_check_maxtracks(struct song *song)
 {
-  int i, oldmax, max = 0;
+  nullcheck_int(song, song_check_maxtracks);
 
-  if (song == NULL) {
-    fprintf(stderr, "song_check_maxtracks() called with null song\n");
-    return 0;
-  }
+  int i, oldmax, max = 0;
 
   oldmax = song->maxtracks;
 
@@ -428,65 +392,78 @@ int song_check_maxtracks(struct song *song)
     if (song->blocks[i]->tracks > max)
       max = song->blocks[i]->tracks;
 
-  /* If the maximum number of tracks as changed... */
-  if (oldmax != max) {
-    /* Reallocate volume array */
-    song->trackvolumes = (unsigned int *)realloc(song->trackvolumes, max * sizeof(unsigned int));
+  if (oldmax == max)
+    /* Do nothing if the maximum number of tracks has not changed */
+    return 0;
+  else if (oldmax < max) {
+    /* New tracks needed: reallocate tracks array */
+    song->tracks = (struct track **)realloc(song->tracks, max * sizeof(struct track *));
 
     /* Set new tracks to volume 127 */
-    for (i = song->maxtracks; i < max; i++)
-      song->trackvolumes[i] = 127;
+    for (i = oldmax; i < max; i++) {
+      /* Give a descriptive name for the new track */
+      char name[10];
+      snprintf(name, 10, "Track %d", i + 1);
+      song->tracks[i] = (struct track *)calloc(1, sizeof(struct track));
+      song->tracks[i]->volume = 127;
+      song->tracks[i]->name = strdup(name);
+    }
+  } else {
+    /* Tracks removed: free track datas */
+    for (i = max; i < oldmax; i++) {
+      if (song->tracks[i]->name != NULL)
+        free (song->tracks[i]->name);
+      free (song->tracks[i]);
+    }
 
-    song->maxtracks = max;
-    return 1;
-  } else
-    return 0;
+    /* Reallocate tracks array */
+    song->tracks = (struct track **)realloc(song->tracks, max * sizeof(struct track *));
+  }
+  
+  song->maxtracks = max;
+  return 1;
 }
 
 /* Make sure the instrument exists; add instruments if necessary */
-void song_check_instrument(struct song *song, int instrument)
+void song_check_instrument(struct song *song, int instrument, unsigned short defaultmidiinterface)
 {
-  if (song == NULL) {
-    fprintf(stderr, "song_check_instrument() called with null song\n");
-    return;
-  }
+  nullcheck_void(song, song_check_instrument);
 
   if (instrument >= song->numinstruments || song->instruments[instrument] == NULL) {
     if (instrument >= song->numinstruments) {
       int i;
 
       song->instruments = realloc(song->instruments, (instrument + 1) * sizeof(struct instrument *));
-      for (i = song->numinstruments; i < (instrument + 1); i++)
+      for (i = song->numinstruments; i < (instrument + 1); i++) {
 	song->instruments[i] = instrument_alloc();
+	song->instruments[i]->outputs[0]->midiinterface = defaultmidiinterface;
+      }
 
       song->numinstruments = instrument + 1;
+    } else {
+      song->instruments[instrument] = instrument_alloc();
+      song->instruments[instrument]->outputs[0]->midiinterface = defaultmidiinterface;
     }
   }
 }
 
 /* Transposes all blocks in a song */
-void song_transpose(struct song *song, int instruments, int halfnotes)
+void song_transpose(struct song *song, int instrument, int halfnotes)
 {
+  nullcheck_void(song, song_transpose);
+
   int i;
 
-  if (song == NULL) {
-    fprintf(stderr, "song_transpose() called with null song\n");
-    return;
-  }
-
   for (i = 0; i < song->numblocks; i++)
-    block_transpose(song->blocks[i], instruments, halfnotes, 0, 0, song->blocks[i]->tracks - 1, song->blocks[i]->length - 1);
+    block_transpose(song->blocks[i], instrument, halfnotes, 0, 0, song->blocks[i]->tracks - 1, song->blocks[i]->length - 1);
 }
 
 /* Expands/shrinks all blocks in a song */
 void song_expandshrink(struct song *song, int factor)
 {
-  int i;
+  nullcheck_void(song, song_expandshrink);
 
-  if (song == NULL) {
-    fprintf(stderr, "song_expandshrink() called with null song\n");
-    return;
-  }
+  int i;
 
   for (i = 0; i < song->numblocks; i++)
     block_expandshrink(song->blocks[i], factor, 0, 0, song->blocks[i]->tracks - 1, song->blocks[i]->length - 1);
@@ -495,12 +472,9 @@ void song_expandshrink(struct song *song, int factor)
 /* Changes an instrument to another in all blocks of a song */
 void song_changeinstrument(struct song *song, int from, int to, int swap)
 {
-  int i;
+  nullcheck_void(song, song_changeinstrument);
 
-  if (song == NULL) {
-    fprintf(stderr, "song_changeinstrument() called with null song\n");
-    return;
-  }
+  int i;
 
   for (i = 0; i < song->numblocks; i++)
     block_changeinstrument(song->blocks[i], from, to, swap, 0, 0, song->blocks[i]->tracks - 1, song->blocks[i]->length - 1);
@@ -668,10 +642,54 @@ struct song *song_parse(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
 	    }
 	  }
 
+	  temp = temp->next;
+	}
+      } else if ((!xmlStrcmp(cur->name, "tracks")) && (cur->ns == ns)) {
+	/* Parse and add all track elements */
+	xmlNodePtr temp = cur->xmlChildrenNode;
+
+	while (temp != NULL) {
+	  if (!(xmlIsBlankNode(temp))) {
+	    int track = -1;
+
+	    /* The track number is required */
+	    if ((!xmlStrcmp(temp->name, "track")) && (temp->ns == ns)) {
+	      xmlNodePtr temp2;
+	      prop = xmlGetProp(temp, "number");
+	      if (prop != NULL)
+		track = atoi(prop);
+
+	      if (track >= song->maxtracks) {
+		int i;
+
+		song->tracks = (struct track **)realloc(song->tracks, (track + 1) * sizeof(struct track *));
+		for (i = song->maxtracks; i <= track; i++)
+                  song->tracks[i] = (struct track *)calloc(1, sizeof(struct track));
+	      }
+
+	      /* Get volume, mute, solo and name */
+	      prop = xmlGetProp(temp, "volume");
+	      if (prop != NULL)
+		song->tracks[track]->volume = atoi(prop);
+	      prop = xmlGetProp(temp, "mute");
+	      if (prop != NULL)
+		song->tracks[track]->mute = atoi(prop);
+	      prop = xmlGetProp(temp, "solo");
+	      if (prop != NULL)
+		song->tracks[track]->solo = atoi(prop);
+
+	      temp2 = temp->xmlChildrenNode;
+	      if (temp2 != NULL)
+		song->tracks[track]->name = strdup(temp2->content);
+	      song->maxtracks = track + 1;
+	    } else if (temp->type != XML_COMMENT_NODE)
+	      fprintf(stderr, "XML error: expected section, got %s\n", temp->name);
+	  }
+
 	  temp = temp->next;
 	}
       } else if ((!xmlStrcmp(cur->name, "trackvolumes")) && (cur->ns == ns)) {
-	/* Parse and add all track volume elements */
+	/* Backwards compatibility: parse and add all track volume elements */
 	xmlNodePtr temp = cur->xmlChildrenNode;
 
 	while (temp != NULL) {
@@ -681,22 +699,24 @@ struct song *song_parse(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
 	    /* The track number is required */
 	    if ((!xmlStrcmp(temp->name, "trackvolume")) && (temp->ns == ns)) {
 	      xmlNodePtr temp2;
-	      prop = xmlGetProp(temp, "track");
+              prop = xmlGetProp(temp, "track");
 	      if (prop != NULL)
 		track = atoi(prop);
 
 	      if (track >= song->maxtracks) {
 		int i;
 
-		song->trackvolumes = realloc(song->trackvolumes, (track + 1) * sizeof(unsigned int));
+		song->tracks = (struct track **)realloc(song->tracks, (track + 1) * sizeof(struct track *));
 		for (i = song->maxtracks; i <= track; i++)
-		  song->trackvolumes[i] = 0;
+                  song->tracks[i] = (struct track *)calloc(1, sizeof(struct track));
 	      }
 
 	      /* Get the volume */
 	      temp2 = temp->xmlChildrenNode;
-	      if (temp2 != NULL)
-		song->trackvolumes[track] = atoi(temp2->content);
+	      if (temp2 != NULL) {
+		song->tracks[track]->volume = (atoi(temp2->content) & 127);
+		song->tracks[track]->mute = (atoi(temp2->content) & 128) >> 7;
+              }
 	      song->maxtracks = track + 1;
 	    } else if (temp->type != XML_COMMENT_NODE)
 	      fprintf(stderr, "XML error: expected section, got %s\n", temp->name);
@@ -748,16 +768,13 @@ struct song *song_parse(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
 /* Saves a song to an XML file */
 void song_save(struct song *song, char *filename)
 {
+  nullcheck_void(song, song_save);
+
   xmlDocPtr doc;
   xmlNodePtr node, subnode;
   int i;
   char c[10];
 
-  if (song == NULL) {
-    fprintf(stderr, "song_save() called with null song\n");
-    return;
-  }
-
   doc = xmlNewDoc("1.0");
   doc->xmlRootNode = xmlNewDocNode(doc, NULL, "song", NULL);
   xmlSetProp(doc->xmlRootNode, "name", song->name);
@@ -804,14 +821,19 @@ void song_save(struct song *song, char *filename)
     instrument_save(song->instruments[i], i, node);
   xmlAddChild(doc->xmlRootNode, xmlNewText("\n\n"));
 
-  node = xmlNewChild(doc->xmlRootNode, NULL, "trackvolumes", NULL);
+  node = xmlNewChild(doc->xmlRootNode, NULL, "tracks", NULL);
   xmlAddChild(node, xmlNewText("\n"));
-  /* Add all track volumes */
+  /* Add all tracks */
   for (i = 0; i < song->maxtracks; i++) {
-    snprintf(c, 10, "%d", song->trackvolumes[i]);
-    subnode = xmlNewChild(node, NULL, "trackvolume", c);
+    subnode = xmlNewChild(node, NULL, "track", song->tracks[i]->name);
     snprintf(c, 10, "%d", i);
-    xmlSetProp(subnode, "track", c);
+    xmlSetProp(subnode, "number", c);
+    snprintf(c, 10, "%d", song->tracks[i]->volume);
+    xmlSetProp(subnode, "volume", c);
+    snprintf(c, 10, "%d", song->tracks[i]->mute);
+    xmlSetProp(subnode, "mute", c);
+    snprintf(c, 10, "%d", song->tracks[i]->solo);
+    xmlSetProp(subnode, "solo", c);
     xmlAddChild(node, xmlNewText("\n"));
   }
   xmlAddChild(doc->xmlRootNode, xmlNewText("\n\n"));
@@ -834,6 +856,8 @@ void song_save(struct song *song, char *filename)
 
 void song_print(struct song *song, char *path)
 {
+  nullcheck_void(song, song_print);
+  
   char *notestrings[] = {
     "---",
     "C-1", "C#1", "D-1", "D#1", "E-1", "F-1", "F#1", "G-1", "G#1", "A-1", "A#1", "B-1",
@@ -860,7 +884,7 @@ void song_print(struct song *song, char *path)
     fprintf(file, "======= Song: %s\n\n", song->name);
     fprintf(file, "#  Instrument name                               Mc Mpst Vol Trns Hold\n");
     for (i = 0; i < song->numinstruments; i++) {
-      fprintf(file, "%.2d %-45s %2d %4d %3d %4d %4d\n", i + 1, song->instruments[i]->name, song->instruments[i]->midichannel, song->instruments[i]->midipreset, song->instruments[i]->defaultvelocity, song->instruments[i]->transpose, song->instruments[i]->hold);
+      fprintf(file, "%.2d %-45s %2d %4d %3d %4d %4d\n", i + 1, song->instruments[i]->name, song->instruments[i]->outputs[0]->midichannel, song->instruments[i]->outputs[0]->midipreset, song->instruments[i]->outputs[0]->defaultvelocity, song->instruments[i]->outputs[0]->transpose, song->instruments[i]->outputs[0]->hold);
     }
 
     fprintf(file, "\nDefault Tempo = %d/%d\n\n", song->tempo, song->ticksperline);
@@ -868,7 +892,7 @@ void song_print(struct song *song, char *path)
     for (i = 0; i < song->maxtracks; i++) {
       if (!(i % 8))
 	fprintf(file, "\n");
-      fprintf(file, "%2d: %3d  ", i + 1, song->trackvolumes[i]);
+      fprintf(file, "%2d: %3d  ", i + 1, song->tracks[i]->volume);
     }
 
     fprintf(file, "\n\n======================================================================\n\n");
