@@ -24,7 +24,7 @@
 #endif
 
 #include <signal.h>
-#include <unistd.h>
+#include <time.h>
 #include <glib.h>
 #include "tutka.h"
 #include "editor.h"
@@ -119,8 +119,8 @@ void editor_song_unlock(struct editor *editor)
 
 int main(int argc, char *argv[])
 {
-  int i;
-  
+  const struct timespec nobusywait = { 0, 40000 };
+
   /* Check command line arguments */
   if (argc < 2) {
     fprintf(stderr, "%s: Not enough arguments\n", argv[0]);
@@ -144,15 +144,11 @@ int main(int argc, char *argv[])
   editor->song = song_load(argv[1]);
 
   if (editor->song != NULL) {
-    /* Get MIDI interfaces */
-    editor->nummidiinterfaces = midi_get_num_alsa_devices();
-    editor->midiinterfaces = midi_get_alsa_devices();
-    
     /* Open MIDI */
-    midi_open_defined(editor->midiinterfaces, editor->nummidiinterfaces, gconf_client_get_string(editor->gconfclient, "/apps/tutka/midiinterfaces/in", NULL), gconf_client_get_string(editor->gconfclient, "/apps/tutka/midiinterfaces/out", NULL), gconf_client_get_string(editor->gconfclient, "/apps/tutka/midiinterfaces/unavailablein", NULL), gconf_client_get_string(editor->gconfclient, "/apps/tutka/midiinterfaces/unavailableout", NULL));
+    editor->midi = midi_open(gconf_client_get_string(editor->gconfclient, "/apps/tutka/midiinterfaces/in", NULL), gconf_client_get_string(editor->gconfclient, "/apps/tutka/midiinterfaces/out", NULL), gconf_client_get_string(editor->gconfclient, "/apps/tutka/midiinterfaces/unavailablein", NULL), gconf_client_get_string(editor->gconfclient, "/apps/tutka/midiinterfaces/unavailableout", NULL));
 
     /* Open the player and start playing */
-    editor->player = player_open(editor->song, editor, editor->midiinterfaces, editor->nummidiinterfaces, gconf_client_get_int(editor->gconfclient, "/apps/tutka/scheduler", NULL));
+    editor->player = player_open(editor->song, editor, editor->midi, gconf_client_get_int(editor->gconfclient, "/apps/tutka/scheduler", NULL));
     editor->player->sched = SCHED_RTC;
     player_start(editor->player, MODE_PLAY_SONG, 0, 0, 0, 0);
 
@@ -163,18 +159,16 @@ int main(int argc, char *argv[])
 	fflush(stdout);
 	statuschanged = 0;
       }
-      usleep(40);
+
+      /* Sleep for a while */
+      nanosleep(&nobusywait, 0);
     }
 
     /* Close the player */
     player_close(editor->player);
 
     /* Close MIDI */
-    if (editor->midiinterfaces != NULL) {
-      for (i = 0; i < editor->nummidiinterfaces; i++)
-        midi_free(editor->midiinterfaces[i]);
-      free(editor->midiinterfaces);
-    }
+    midi_close(editor->midi);
     
     song_free(editor->song);
   }
